@@ -3,7 +3,7 @@
  * functions, so that module test harnesses can run standalone.
  *
  * Copyright (C) 2001, 2002 Martin Pool <mbp@samba.org>
- * Copyright (C) 2003-2014 Wayne Davison
+ * Copyright (C) 2003-2020 Wayne Davison
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,16 +21,19 @@
 
 #include "rsync.h"
 
+int inplace = 0;
 int modify_window = 0;
 int preallocate_files = 0;
 int protect_args = 0;
 int module_id = -1;
-int checksum_len = 0;
 int relative_paths = 0;
 int module_dirlen = 0;
-int preserve_acls = 0;
 int preserve_times = 0;
 int preserve_xattrs = 0;
+int preserve_perms = 0;
+int preserve_executability = 0;
+int open_noatime = 0;
+size_t max_alloc = 1024*1024*1024; /* max_alloc is needed when combined with util2.o */
 char *partial_dir;
 char *module_dir;
 filter_rule_list daemon_filter_list;
@@ -97,3 +100,13 @@ filter_rule_list daemon_filter_list;
 {
 	return "tester";
 }
+
+ int csum_len_for_type(int cst, int flg)
+{
+	return cst || !flg ? 16 : 1;
+}
+
+ int canonical_checksum(int cst)
+{
+	return cst ? 0 : 0;
+}
